@@ -1,4 +1,4 @@
-/* $Id: shutdown.c 28621 2008-11-06 07:34:07Z kelnos $ */
+/* $Id$ */
 /*-
  * Copyright (c) 2003-2004 Benedikt Meurer <benny@xfce.org>
  * All rights reserved.
@@ -40,12 +40,20 @@
 #include <unistd.h>
 #endif
 
+#ifdef HAVE_GETPWUID
+#ifdef HAVE_SYS_TYPES_H
+#include <sys/types.h>
+#endif
+#ifdef HAVE_PWD_H
+#include <pwd.h>
+#endif
+#endif
+
 #ifdef HAVE_ASM_UNISTD_H
 #include <asm/unistd.h>  /* for __NR_ioprio_set */
 #endif
 
 #include <libxfce4util/libxfce4util.h>
-#include <libxfcegui4/libxfcegui4.h>
 #include <gtk/gtk.h>
 
 #include <libxfsm/xfsm-util.h>
@@ -88,7 +96,7 @@ screenshot_save (const gchar *session_name, GdkPixmap *pm, GdkRectangle *area)
         {
           /* determine thumb file */
           dpy = gdk_drawable_get_display (GDK_DRAWABLE (pm));
-          display_name = xfce_gdk_display_get_fullname (dpy);
+          display_name = xfsm_gdk_display_get_fullname (dpy);
           resource = g_strconcat ("sessions/thumbs-", display_name,
                                   "/", session_name, ".png", NULL);
           filename = xfce_resource_save_location (XFCE_RESOURCE_CACHE,
@@ -160,6 +168,7 @@ gboolean
 shutdownDialog(const gchar *sessionName, XfsmShutdownType *shutdownType, gboolean *saveSession)
 {
   gboolean accessibility;
+  GtkIconTheme *icon_theme;
   XfsmFadeout *fadeout = NULL;
   GdkScreen *screen;
   GtkWidget *dialog;
@@ -186,6 +195,12 @@ shutdownDialog(const gchar *sessionName, XfsmShutdownType *shutdownType, gboolea
   gboolean prompt;
   gboolean show_suspend;
   gboolean show_hibernate;
+  
+  gboolean show_restart;
+  gboolean show_shutdown;
+
+  gboolean require_password;
+
   gint monitor;
   gint result;
   XfceKiosk *kiosk;
@@ -198,14 +213,19 @@ shutdownDialog(const gchar *sessionName, XfsmShutdownType *shutdownType, gboolea
   GdkPixmap *screenshot_pm = NULL;
   GdkGC *screenshot_gc;
 #endif
+#ifdef HAVE_GETPWUID
+  struct passwd *pw;
+#endif
 
   g_return_val_if_fail(saveSession != NULL, FALSE);
   g_return_val_if_fail(shutdownType != NULL, FALSE);
 
+  icon_theme = gtk_icon_theme_get_default ();
+
   /* destroy any previously running shutdown helper first */
   if (shutdown_helper != NULL)
     {
-      xfsm_shutdown_helper_destroy (shutdown_helper);
+      g_object_unref (shutdown_helper);
       shutdown_helper = NULL;
     }
 
@@ -240,6 +260,10 @@ shutdownDialog(const gchar *sessionName, XfsmShutdownType *shutdownType, gboolea
       return TRUE;
     }
 
+  /* spawn the helper early so we know what it supports when
+   * constructing the dialog */
+  shutdown_helper = xfsm_shutdown_helper_new ();
+
   /* It's really bad here if someone else has the pointer
    * grabbed, so we first grab the pointer and keyboard
    * to an offscreen window, and then once we have the
@@ -248,7 +272,7 @@ shutdownDialog(const gchar *sessionName, XfsmShutdownType *shutdownType, gboolea
   gtk_rc_reparse_all ();
 
   /* get screen with pointer */
-  screen = xfce_gdk_display_locate_monitor_with_pointer (NULL, &monitor);
+  screen = xfce_gdk_screen_get_active (&monitor);
   if (screen == NULL)
     {
       screen = gdk_screen_get_default ();
@@ -341,6 +365,26 @@ shutdownDialog(const gchar *sessionName, XfsmShutdownType *shutdownType, gboolea
   gtk_container_set_border_width (GTK_CONTAINER (vbox), BORDER);
   gtk_widget_show(vbox);
 
+#ifdef HAVE_GETPWUID
+  pw = getpwuid (getuid ());
+  if (G_LIKELY(pw && pw->pw_name && *pw->pw_name))
+    {
+      gchar *text = g_strdup_printf (_("<span size='large'><b>Log out %s</b></span>"), pw->pw_name);
+      GtkWidget *logout_label = g_object_new (GTK_TYPE_LABEL,
+                                             "label", text,
+                                             "use-markup", TRUE,
+                                             "justify", GTK_JUSTIFY_CENTER,
+                                             "xalign", 0.5,
+                                             "yalign", 0.5,
+                                             NULL);
+
+      gtk_widget_show (logout_label);
+      gtk_box_pack_start (GTK_BOX (vbox), logout_label, FALSE, FALSE, 0);
+
+      g_free (text);
+    }
+#endif
+
   hbox = gtk_hbox_new (TRUE, BORDER);
   gtk_widget_show (hbox);
   gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);
@@ -358,9 +402,18 @@ shutdownDialog(const gchar *sessionName, XfsmShutdownType *shutdownType, gboolea
   gtk_widget_show (vbox2);
   gtk_container_add (GTK_CONTAINER (logout_button), vbox2);
 
-  icon = xfce_themed_icon_load ("system-log-out", 32);
+  icon = gtk_icon_theme_load_icon (icon_theme,
+                                   "system-log-out",
+                                   32,
+                                   GTK_ICON_LOOKUP_GENERIC_FALLBACK,
+                                   NULL);
   if (!icon)
-    icon = xfce_themed_icon_load ("xfsm-logout", 32);
+    icon = gtk_icon_theme_load_icon (icon_theme,
+                                     "xfsm-logout",
+                                     32,
+                                     GTK_ICON_LOOKUP_GENERIC_FALLBACK,
+                                     NULL);
+
   image = gtk_image_new_from_pixbuf (icon);
   gtk_widget_show (image);
   gtk_box_pack_start (GTK_BOX (vbox2), image, FALSE, FALSE, 0);
@@ -383,9 +436,19 @@ shutdownDialog(const gchar *sessionName, XfsmShutdownType *shutdownType, gboolea
   gtk_widget_show (vbox2);
   gtk_container_add (GTK_CONTAINER (reboot_button), vbox2);
 
-  icon = xfce_themed_icon_load ("system-reboot", 32);
+  icon = gtk_icon_theme_load_icon (icon_theme,
+                                   "system-reboot",
+                                   32,
+                                   GTK_ICON_LOOKUP_GENERIC_FALLBACK,
+                                   NULL);
+ 
   if (!icon)
-    icon = xfce_themed_icon_load ("xfsm-reboot", 32);
+    icon = gtk_icon_theme_load_icon (icon_theme,
+                                     "xfsm-reboot",
+                                     32,
+                                     GTK_ICON_LOOKUP_GENERIC_FALLBACK,
+                                     NULL);
+ 
   image = gtk_image_new_from_pixbuf (icon);
   gtk_widget_show (image);
   gtk_box_pack_start (GTK_BOX (vbox2), image, FALSE, FALSE, 0);
@@ -394,6 +457,13 @@ shutdownDialog(const gchar *sessionName, XfsmShutdownType *shutdownType, gboolea
   label = gtk_label_new (_("Restart"));
   gtk_widget_show (label);
   gtk_box_pack_start (GTK_BOX (vbox2), label, FALSE, FALSE, 0);
+
+  g_object_get (shutdown_helper,
+		"user-can-restart", &show_restart,
+		NULL);
+
+  if (!kiosk_can_shutdown || !show_restart )
+    gtk_widget_set_sensitive (reboot_button, FALSE);
   
   /* halt */
   halt_button = gtk_button_new ();
@@ -408,19 +478,47 @@ shutdownDialog(const gchar *sessionName, XfsmShutdownType *shutdownType, gboolea
   gtk_widget_show (vbox2);
   gtk_container_add (GTK_CONTAINER (halt_button), vbox2);
 
-  icon = xfce_themed_icon_load ("system-shutdown", 32);
+  icon = gtk_icon_theme_load_icon (icon_theme,
+                                   "system-shutdown",
+                                   32,
+                                   GTK_ICON_LOOKUP_GENERIC_FALLBACK,
+                                   NULL);
+
   if (!icon)
-    icon = xfce_themed_icon_load ("xfsm-shutdown", 32);
+    icon = gtk_icon_theme_load_icon (icon_theme,
+                                     "xfsm-shutdown",
+                                     32,
+                                     GTK_ICON_LOOKUP_GENERIC_FALLBACK,
+                                     NULL);
+
   image = gtk_image_new_from_pixbuf (icon);
   gtk_widget_show (image);
   gtk_box_pack_start (GTK_BOX (vbox2), image, FALSE, FALSE, 0);
   g_object_unref (icon);
-  
+
   label = gtk_label_new (_("Shut Down"));
   gtk_widget_show (label);
   gtk_box_pack_start (GTK_BOX (vbox2), label, FALSE, FALSE, 0);
-  
-  if (show_suspend || show_hibernate)
+
+  g_object_get (shutdown_helper, 
+		"user-can-shutdown", &show_shutdown,
+		NULL);
+
+  if (!kiosk_can_shutdown || !show_shutdown)
+    gtk_widget_set_sensitive (halt_button, FALSE);
+
+  if (show_suspend)
+    g_object_get (shutdown_helper,
+		  "user-can-suspend", &show_suspend,
+		  NULL);
+
+  if (show_hibernate)
+    g_object_get (shutdown_helper,
+		  "user-can-hibernate", &show_hibernate,
+		  NULL);
+
+
+  if (kiosk_can_shutdown && (show_suspend || show_hibernate))
     {
       hbox = gtk_hbox_new (FALSE, BORDER);
       gtk_widget_show (hbox);
@@ -428,7 +526,7 @@ shutdownDialog(const gchar *sessionName, XfsmShutdownType *shutdownType, gboolea
     }
 
   /* suspend */
-  if (show_suspend)
+  if (kiosk_can_shutdown && show_suspend)
     {
       suspend_button = gtk_button_new ();
       gtk_widget_show (suspend_button);
@@ -442,9 +540,19 @@ shutdownDialog(const gchar *sessionName, XfsmShutdownType *shutdownType, gboolea
       gtk_widget_show (vbox2);
       gtk_container_add (GTK_CONTAINER (suspend_button), vbox2);
 
-      icon = xfce_themed_icon_load ("system-suspend", 32);
+      icon = gtk_icon_theme_load_icon (icon_theme,
+                                       "system-suspend",
+                                       32,
+                                       GTK_ICON_LOOKUP_GENERIC_FALLBACK,
+                                       NULL);
+
       if (!icon)
-        icon = xfce_themed_icon_load ("xfsm-suspend", 32);
+        icon = gtk_icon_theme_load_icon (icon_theme,
+                                         "xfsm-suspend",
+                                         32,
+                                         GTK_ICON_LOOKUP_GENERIC_FALLBACK,
+                                         NULL);
+
       image = gtk_image_new_from_pixbuf (icon);
       gtk_widget_show (image);
       gtk_box_pack_start (GTK_BOX (vbox2), image, FALSE, FALSE, 0);
@@ -456,7 +564,7 @@ shutdownDialog(const gchar *sessionName, XfsmShutdownType *shutdownType, gboolea
     }
 
   /* hibernate */
-  if (show_hibernate)
+  if (kiosk_can_shutdown && show_hibernate)
     {
       hibernate_button = gtk_button_new ();
       gtk_widget_show (hibernate_button);
@@ -470,9 +578,19 @@ shutdownDialog(const gchar *sessionName, XfsmShutdownType *shutdownType, gboolea
       gtk_widget_show (vbox2);
       gtk_container_add (GTK_CONTAINER (hibernate_button), vbox2);
 
-      icon = xfce_themed_icon_load ("system-hibernate", 32);
+      icon = gtk_icon_theme_load_icon (icon_theme,
+                                       "system-hibernate",
+                                       32,
+                                       GTK_ICON_LOOKUP_GENERIC_FALLBACK,
+                                       NULL);
+
       if (!icon)
-        icon = xfce_themed_icon_load ("xfsm-hibernate", 32);
+        icon = gtk_icon_theme_load_icon (icon_theme,
+                                         "xfsm-hibernate",
+                                         32,
+                                         GTK_ICON_LOOKUP_GENERIC_FALLBACK,
+                                         NULL);
+
       image = gtk_image_new_from_pixbuf (icon);
       gtk_widget_show (image);
       gtk_box_pack_start (GTK_BOX (vbox2), image, FALSE, FALSE, 0);
@@ -500,21 +618,10 @@ shutdownDialog(const gchar *sessionName, XfsmShutdownType *shutdownType, gboolea
   /* create small border */
   if (!accessibility)
     xfsm_window_add_border (GTK_WINDOW (dialog));
-  
-  /* center dialog on target monitor */
-  xfce_gtk_window_center_on_monitor (GTK_WINDOW (dialog), screen, monitor);
 
-  /* connect to the shutdown helper */
-  if (!kiosk_can_shutdown || 
-      (shutdown_helper = xfsm_shutdown_helper_spawn (NULL)) == NULL)
-    {
-      gtk_widget_set_sensitive (reboot_button, FALSE);
-      gtk_widget_set_sensitive (halt_button, FALSE);
-      if (suspend_button)
-        gtk_widget_set_sensitive (suspend_button, FALSE);
-      if (hibernate_button)
-        gtk_widget_set_sensitive (hibernate_button, FALSE);
-    }
+  /* center dialog on target monitor */
+  gtk_window_set_screen (GTK_WINDOW (dialog), screen);
+  gtk_window_set_position (GTK_WINDOW (dialog), GTK_WIN_POS_CENTER);
 
   /* save portion of the root window covered by the dialog */
   if (!accessibility && shutdown_helper != NULL)
@@ -542,9 +649,13 @@ shutdownDialog(const gchar *sessionName, XfsmShutdownType *shutdownType, gboolea
 
   gtk_widget_hide (dialog);
 
+  g_object_get (shutdown_helper,
+		"require-password", &require_password,
+		NULL);
+
   /* ask password */
   if (result == GTK_RESPONSE_OK && *shutdownType != XFSM_SHUTDOWN_LOGOUT
-      && xfsm_shutdown_helper_need_password (shutdown_helper))
+      && require_password )
     {
       gtk_widget_show (ok_button);
 
@@ -567,7 +678,8 @@ shutdownDialog(const gchar *sessionName, XfsmShutdownType *shutdownType, gboolea
       gtk_widget_show (entry);
 
       /* center dialog on target monitor */
-      xfce_gtk_window_center_on_monitor (GTK_WINDOW (dialog), screen, monitor);
+      gtk_window_set_screen (GTK_WINDOW (dialog), screen);
+      gtk_window_set_position (GTK_WINDOW (dialog), GTK_WIN_POS_CENTER);
 
       gtk_widget_show_now (dialog);
       gtk_widget_grab_focus (entry);
@@ -580,9 +692,9 @@ shutdownDialog(const gchar *sessionName, XfsmShutdownType *shutdownType, gboolea
 
       if (result == GTK_RESPONSE_OK)
         {
-          const gchar *pw = gtk_entry_get_text (GTK_ENTRY (entry));
+          const gchar *password = gtk_entry_get_text (GTK_ENTRY (entry));
 
-          if (!xfsm_shutdown_helper_send_password (shutdown_helper, pw))
+          if (!xfsm_shutdown_helper_send_password (shutdown_helper, password))
             {
               gtk_label_set_text (GTK_LABEL (label),
                                   _("<b>An error occurred</b>"));
@@ -608,8 +720,8 @@ shutdownDialog(const gchar *sessionName, XfsmShutdownType *shutdownType, gboolea
               gtk_widget_show (label);
 
               /* center dialog on target monitor */
-              xfce_gtk_window_center_on_monitor (GTK_WINDOW (dialog),
-                                                 screen, monitor);
+              gtk_window_set_screen (GTK_WINDOW (dialog), screen);
+              gtk_window_set_position (GTK_WINDOW (dialog), GTK_WIN_POS_CENTER);
 
               gtk_widget_show_now (dialog);
 
@@ -653,7 +765,7 @@ shutdownDialog(const gchar *sessionName, XfsmShutdownType *shutdownType, gboolea
     }
   else
     {
-      xfsm_shutdown_helper_destroy (shutdown_helper);
+      g_object_unref (shutdown_helper);
       shutdown_helper = NULL;
     }
 
@@ -716,7 +828,7 @@ xfsm_shutdown(XfsmShutdownType type)
     }
 
   result = xfsm_shutdown_helper_send_command (shutdown_helper, type, &error);
-  xfsm_shutdown_helper_destroy (shutdown_helper);
+  g_object_unref (shutdown_helper);
   shutdown_helper = NULL;
 
   if (!result)
