@@ -1,4 +1,4 @@
-/* $Id: shutdown.h 20833 2006-04-09 18:08:17Z benny $ */
+/* $Id$ */
 /*-
  * Copyright (c) 2003-2004 Benedikt Meurer <benny@xfce.org>
  * All rights reserved.
@@ -24,13 +24,13 @@
 
 #include <glib.h>
 
-/* */
-#define	SHUTDOWN_LOGOUT	0
-#define SHUTDOWN_REBOOT	1
-#define SHUTDOWN_HALT	2
+#include "xfsm-global.h"
 
 /* prototypes */
-extern gboolean	shutdownDialog(gint *, gboolean *);
-extern gint	xfsm_shutdown(gint);
+gboolean     shutdownDialog   (const gchar *sessionName,
+                               XfsmShutdownType *shutdownType,
+                               gboolean *saveSession);
+
+gint	     xfsm_shutdown    (XfsmShutdownType type);
 
 #endif	/* !__XFSM_SHUTDOWN_H__ */
