@@ -43,7 +43,7 @@
 #endif
 
 #include <libxfce4util/libxfce4util.h>
-#include <libxfcegui4/libxfcegui4.h>
+#include <libxfce4ui/libxfce4ui.h>
 
 #include <xfce4-session/shutdown.h>
 #include <xfce4-session/xfsm-global.h>
@@ -79,6 +79,7 @@ xfsm_verbose_real (const gchar *format, ...)
   static FILE *fp = NULL;
   gchar       *logfile;
   va_list      valist;
+  GTimeVal     tv;
   
   if (G_UNLIKELY (fp == NULL))
     {
@@ -86,6 +87,9 @@ xfsm_verbose_real (const gchar *format, ...)
       fp = fopen (logfile, "w");
       g_free (logfile);
     }
+
+  g_get_current_time(&tv);
+  fprintf(fp, "[%10lu] ", tv.tv_sec);
   
   va_start (valist, format);
   vfprintf (fp, format, valist);
@@ -146,7 +150,7 @@ xfsm_load_session_preview (const gchar *name)
 
   /* determine thumb file */
   display = gdk_display_get_default ();
-  display_name = xfce_gdk_display_get_fullname (display);
+  display_name = xfsm_gdk_display_get_fullname (display);
   resource = g_strconcat ("sessions/thumbs-", display_name,
                           "/", name, ".png", NULL);
   filename = xfce_resource_save_location (XFCE_RESOURCE_CACHE, resource, TRUE);
@@ -182,3 +186,4 @@ xfsm_g_value_free (GValue *value)
       g_free (value);
     }
 }
+
