@@ -1,4 +1,4 @@
-/* $Id: xfsm-global.h 20833 2006-04-09 18:08:17Z benny $ */
+/* $Id$ */
 /*-
  * Copyright (c) 2003-2004 Benedikt Meurer <benny@xfce.org>
  * All rights reserved.
@@ -22,8 +22,22 @@
 #ifndef __XFSM_GLOBAL_H__
 #define __XFSM_GLOBAL_H__
 
-#include <xfce4-session/xfsm-splash-screen.h>
+#include <glib.h>
 
+#include <X11/SM/SMlib.h>
+
+#include <xfce4-session/xfsm-splash-screen.h>
+#include <dbus/dbus.h>
+
+typedef enum
+{
+  XFSM_SHUTDOWN_ASK = 0,
+  XFSM_SHUTDOWN_LOGOUT,
+  XFSM_SHUTDOWN_HALT,
+  XFSM_SHUTDOWN_REBOOT,
+  XFSM_SHUTDOWN_SUSPEND,
+  XFSM_SHUTDOWN_HIBERNATE,
+} XfsmShutdownType;
 
 typedef struct _FailsafeClient FailsafeClient;
 struct _FailsafeClient
@@ -31,23 +45,14 @@ struct _FailsafeClient
   gchar    **command;
   GdkScreen *screen;
 };
+
+void xfsm_failsafe_client_free (FailsafeClient *fclient);
   
 
 #define DEFAULT_SESSION_NAME "Default"
 
 
 extern gboolean          verbose;
-extern gboolean          compat_kde;
-extern gboolean          compat_gnome;
-extern GList            *starting_properties;
-extern GList            *pending_properties;
-extern GList            *restart_properties;
-extern GList            *running_clients;
-extern gchar            *session_name;
-extern gchar            *session_file;
-extern GList            *failsafe_clients;
-extern gboolean          failsafe_mode;
-extern gint              shutdown_type;
 extern XfsmSplashScreen *splash_screen;
 
 
@@ -72,5 +77,11 @@ G_STMT_START{                         \
 void xfsm_enable_verbose (void);
 void xfsm_verbose_real (const gchar *format, ...) G_GNUC_PRINTF (1, 2);
 
+gchar *xfsm_generate_client_id (SmsConn sms_conn) G_GNUC_PURE;
+
+GdkPixbuf *xfsm_load_session_preview (const gchar *name);
+
+GValue *xfsm_g_value_new (GType gtype);
+void    xfsm_g_value_free (GValue *value);
 
 #endif /* !__XFSM_GLOBAL_H__ */
