@@ -1,4 +1,4 @@
-/* $Id: xfsm-legacy.c 4751 2004-12-27 17:25:51Z benny $ */
+/* $Id$ */
 /*-
  * Copyright (c) 2004 Benedikt Meurer <benny@xfce.org>
  * All rights reserved.
@@ -71,8 +71,11 @@
 
 #include <gdk/gdkx.h>
 
-#include <libxfcegui4/libxfcegui4.h>
+#include <libxfce4ui/libxfce4ui.h>
 
+#include <libwnck/libwnck.h>
+
+#include "xfsm-legacy.h"
 #include <libxfsm/xfsm-util.h>
 
 
@@ -293,7 +296,7 @@ xfsm_legacy_perform_session_save (void)
 {
 #ifdef LEGACY_SESSION_MANAGEMENT
   XErrorHandler old_handler;
-  NetkScreen *screen;
+  WnckScreen *screen;
   GList *windows;
   GList *lp;
   Window leader;
@@ -331,14 +334,14 @@ xfsm_legacy_perform_session_save (void)
   /* query mapped windows on all screens */
   for (n = 0; n < ScreenCount (gdk_display); ++n)
     {
-      screen = netk_screen_get (n);
-      netk_screen_force_update (screen);
+      screen = wnck_screen_get (n);
+      wnck_screen_force_update (screen);
 
-      windows = netk_screen_get_windows (screen);
+      windows = wnck_screen_get_windows (screen);
 
       for (lp = windows; lp != NULL; lp = lp->next)
         {
-          window = netk_window_get_xid (NETK_WINDOW (lp->data));
+          window = wnck_window_get_xid (WNCK_WINDOW (lp->data));
           leader = get_clientleader (window);
           if (leader == None || sm_window_list_contains (leader)
               || has_xsmp_support (window) || has_xsmp_support (leader))
@@ -578,9 +581,9 @@ xfsm_legacy_init (void)
     {
       root = RootWindow (dpy, n);
       XChangeProperty (dpy, root, dt_save_mode, XA_STRING, 8,
-                       PropModeReplace, "xfce4", sizeof ("xfce4"));
+                       PropModeReplace, (unsigned char *)"xfce4", sizeof ("xfce4"));
       XChangeProperty (dpy, root, dt_restore_mode, XA_STRING, 8,
-                       PropModeReplace, "xfce4", sizeof ("xfce4"));
+                       PropModeReplace, (unsigned char *)"xfce4", sizeof ("xfce4"));
     }
 #endif
 }
