@@ -74,7 +74,7 @@
 #include <libwnck/libwnck.h>
 
 #include "xfsm-shutdown-helper.h"
-#include <libxfcegui4/libxfcegui4.h>
+#include <libxfce4ui/libxfce4ui.h>
 
 #include <libxfsm/xfsm-splash-engine.h>
 #include <libxfsm/xfsm-util.h>
@@ -117,7 +117,14 @@ struct _XfsmManager
 
   guint            die_timeout_id;
 
-  DBusGConnection *dbus_conn;
+  DBusGConnection *session_bus;
+
+
+#ifdef ENABLE_CONSOLE_KIT
+  DBusGConnection *system_bus;
+  DBusGProxy      *consolekit_proxy;
+  gchar           *consolekit_cookie;
+#endif
 };
 
 typedef struct _XfsmManagerClass
@@ -158,8 +165,6 @@ enum
 };
 
 
-static void       xfsm_manager_class_init (XfsmManagerClass *klass);
-static void       xfsm_manager_init (XfsmManager *manager);
 static void       xfsm_manager_finalize (GObject *obj);
 
 static gboolean   xfsm_manager_startup (XfsmManager *manager);
@@ -175,6 +180,10 @@ static void       xfsm_manager_dbus_class_init (XfsmManagerClass *klass);
 static void       xfsm_manager_dbus_init (XfsmManager *manager);
 static void       xfsm_manager_dbus_cleanup (XfsmManager *manager);
 
+#ifdef ENABLE_CONSOLE_KIT
+static void       xfsm_manager_consolekit_init (XfsmManager *manager);
+static void       xfsm_manager_consolekit_cleanup (XfsmManager *manager);
+#endif /*ENABLE_CONSOLE_KIT*/
 
 static guint signals[N_SIGS] = { 0, };
 
@@ -225,18 +234,40 @@ xfsm_manager_class_init (XfsmManagerClass *klass)
 static void
 xfsm_manager_init (XfsmManager *manager)
 {
+#ifdef ENABLE_CONSOLE_KIT
+  GError *error = NULL;
+#endif
+
   manager->state = XFSM_MANAGER_STARTUP;
   manager->session_chooser = FALSE;
   manager->failsafe_mode = TRUE;
   manager->shutdown_type = XFSM_SHUTDOWN_LOGOUT;
 
+#ifdef ENABLE_CONSOLE_KIT
+  manager->consolekit_proxy  = NULL;
+  manager->consolekit_cookie = NULL;
+#endif /*ENABLE_CONSOLE_KIT*/
+
   manager->pending_properties = g_queue_new ();
   manager->starting_properties = g_queue_new ();
   manager->restart_properties = g_queue_new ();
   manager->running_clients = g_queue_new ();
   manager->failsafe_clients = g_queue_new ();
-}
 
+#ifdef ENABLE_CONSOLE_KIT
+  manager->system_bus = dbus_g_bus_get (DBUS_BUS_SYSTEM, &error);
+  
+  if ( G_LIKELY (manager->system_bus != NULL) )
+    {
+      xfsm_manager_consolekit_init (manager);
+    }
+  else if ( error )
+    {
+      g_warning ("Failed to connect to the system bus : %s", error->message);
+      g_error_free (error);
+    }
+#endif
+}
 
 static void
 xfsm_manager_finalize (GObject *obj)
@@ -245,6 +276,13 @@ xfsm_manager_finalize (GObject *obj)
 
   xfsm_manager_dbus_cleanup (manager);
 
+#ifdef ENABLE_CONSOLE_KIT
+  xfsm_manager_consolekit_cleanup (manager);
+  
+  if ( manager->system_bus )
+    dbus_g_connection_unref (manager->system_bus);
+#endif /*ENABLE_CONSOLE_KIT*/
+
   if (manager->die_timeout_id != 0)
     g_source_remove (manager->die_timeout_id);
 
@@ -304,6 +342,73 @@ xfsm_manager_new (void)
   return manager;
 }
 
+#ifdef ENABLE_CONSOLE_KIT
+static void xfsm_manager_consolekit_init (XfsmManager *manager)
+{
+  GError *error = NULL;
+  gboolean ret;
+  
+  manager->consolekit_proxy = dbus_g_proxy_new_for_name_owner (manager->system_bus,
+							       "org.freedesktop.ConsoleKit",
+							       "/org/freedesktop/ConsoleKit/Manager",
+							       "org.freedesktop.ConsoleKit.Manager",
+							       NULL);
+
+  
+  if ( G_UNLIKELY (!manager->consolekit_proxy) )
+    {
+      g_warning ("Failed to create proxy for 'org.freedesktop.ConsoleKit'");
+      return;
+    }
+
+  ret = dbus_g_proxy_call (manager->consolekit_proxy, "OpenSession", &error,
+			   G_TYPE_INVALID,
+			   G_TYPE_STRING, &manager->consolekit_cookie,
+			   G_TYPE_INVALID);
+
+  if ( G_LIKELY (ret) )
+    {
+      /*
+       * ConsoleKit doc says that the leader session should set the cookie
+       * on XDG_SESSION_COOKIE env variable.
+       */
+      g_warn_if_fail (g_setenv ("XDG_SESSION_COOKIE",
+				manager->consolekit_cookie,
+				TRUE));
+    }
+  else if ( error )
+    {
+      g_warning ("OpenSession on 'org.freedesktop.ConsoleKit' failed with %s", error->message);
+      g_error_free (error);
+    }
+}
+
+static void xfsm_manager_consolekit_cleanup (XfsmManager *manager)
+{
+  if ( manager->consolekit_proxy )
+    {
+      if (manager->consolekit_cookie) 
+	{
+	  GError *error = NULL;
+	  gboolean ret, result;
+	  
+	  ret = dbus_g_proxy_call (manager->consolekit_proxy, "CloseSession", &error,
+				   G_TYPE_STRING, manager->consolekit_cookie,
+				   G_TYPE_INVALID,
+				   G_TYPE_BOOLEAN, &result,
+				   G_TYPE_INVALID);
+	  if ( !ret )
+	    {
+	      g_warning ("CloseSession on 'org.freedesktop.ConsoleKit' failed with %s", error->message);
+	      g_error_free (error);
+	    }
+	  g_free (manager->consolekit_cookie);
+	}
+      g_object_unref (manager->consolekit_proxy);
+    }
+}
+#endif /*ENABLE_CONSOLE_KIT*/
+
 
 static gboolean
 xfsm_manager_startup (XfsmManager *manager)
@@ -349,6 +454,9 @@ gboolean
 xfsm_manager_handle_failed_properties (XfsmManager    *manager,
                                        XfsmProperties *properties)
 {
+  gint restart_style_hint;
+  GError *error = NULL;
+
   /* Handle apps that failed to start, or died randomly, here */
 
   xfsm_properties_set_default_child_watch (properties);
@@ -359,11 +467,15 @@ xfsm_manager_handle_failed_properties (XfsmManager    *manager,
       properties->restart_attempts_reset_id = 0;
     }
 
-  if (properties->restart_style_hint == SmRestartAnyway)
+  restart_style_hint = xfsm_properties_get_uchar (properties,
+                                                  SmRestartStyleHint,
+                                                  SmRestartIfRunning);
+
+  if (restart_style_hint == SmRestartAnyway)
     {
       g_queue_push_tail (manager->restart_properties, properties);
     }
-  else if (properties->restart_style_hint == SmRestartImmediately)
+  else if (restart_style_hint == SmRestartImmediately)
     {
       if (++properties->restart_attempts > MAX_RESTART_ATTEMPTS)
         {
@@ -393,6 +505,8 @@ xfsm_manager_handle_failed_properties (XfsmManager    *manager,
     }
   else
     {
+      gchar **discard_command;
+
       /* We get here if a SmRestartNever or SmRestartIfRunning client
        * has exited.  SmRestartNever clients shouldn't have discard
        * commands, but it can't hurt to run it if it has one for some
@@ -400,7 +514,8 @@ xfsm_manager_handle_failed_properties (XfsmManager    *manager,
       xfsm_verbose ("Client Id %s exited, removing from session.\n",
                     properties->client_id);
 
-      if (properties->discard_command != NULL)
+      discard_command = xfsm_properties_get_strv (properties, SmDiscardCommand);
+      if (discard_command != NULL && g_strv_length (discard_command) > 0)
         {
           /* Run the SmDiscardCommand after the client exited in any state,
            * but only if we don't expect the client to be restarted,
@@ -419,16 +534,22 @@ xfsm_manager_handle_failed_properties (XfsmManager    *manager,
            * ever-growing number of xfwm4 session files when restarting
            * xfwm4 within a session.
            */
-          xfsm_verbose ("Client Id = %s: running discard command.\n\n",
-                        properties->client_id);
+          xfsm_verbose ("Client Id = %s: running discard command %s:%d.\n\n",
+                        properties->client_id, *discard_command,
+                        g_strv_length (discard_command));
 
-          g_spawn_sync (properties->current_directory,
-                        properties->discard_command,
-                        properties->environment,
-                        G_SPAWN_SEARCH_PATH,
-                        NULL, NULL,
-                        NULL, NULL,
-                        NULL, NULL);
+          if (!g_spawn_sync (xfsm_properties_get_string (properties, SmCurrentDirectory),
+                             discard_command,
+                             xfsm_properties_get_strv (properties, SmEnvironment),
+                             G_SPAWN_SEARCH_PATH,
+                             NULL, NULL,
+                             NULL, NULL,
+                             NULL, &error))
+            {
+              g_warning ("Failed to running discard command \"%s\": %s",
+                         *discard_command, error->message);
+              g_error_free (error);
+            }
         }
 
       return FALSE;
@@ -467,8 +588,8 @@ xfsm_manager_choose_session (XfsmManager *manager,
             {
               if (G_UNLIKELY (preview_default == NULL))
                 {
-                  preview_default = gdk_pixbuf_from_pixdata (&chooser_icon_data,
-                                                             FALSE, NULL);
+                  preview_default = gdk_pixbuf_new_from_inline (-1, chooser_icon_data,
+                                                                FALSE, NULL);
                 }
 
               session->preview = GDK_PIXBUF (g_object_ref (preview_default));
@@ -719,7 +840,7 @@ xfsm_manager_load (XfsmManager   *manager,
   manager->compat_gnome = xfconf_channel_get_bool (channel, "/compat/LaunchGNOME", FALSE);
   manager->compat_kde = xfconf_channel_get_bool (channel, "/compat/LaunchKDE", FALSE);
 
-  display_name  = xfce_gdk_display_get_fullname (gdk_display_get_default ());
+  display_name  = xfsm_gdk_display_get_fullname (gdk_display_get_default ());
 
 #ifdef HAVE_OS_CYGWIN
   /* rename a colon (:) to a hash (#) under cygwin. windows doesn't like
@@ -752,7 +873,7 @@ xfsm_manager_restart (XfsmManager *manager)
   /* tell splash screen that the session is starting now */
   preview = xfsm_load_session_preview (manager->session_name);
   if (preview == NULL)
-    preview = gdk_pixbuf_from_pixdata (&chooser_icon_data, FALSE, NULL);
+    preview = gdk_pixbuf_new_from_inline (-1, chooser_icon_data, FALSE, NULL);
   steps = g_queue_get_length (manager->failsafe_mode ? manager->failsafe_clients : manager->pending_properties);
   xfsm_splash_screen_start (splash_screen, manager->session_name, preview, steps);
   g_object_unref (preview);
@@ -967,10 +1088,10 @@ xfsm_manager_interact (XfsmManager *manager,
            lp;
            lp = lp->next)
         {
-          XfsmClient *client = lp->data;
-          if (xfsm_client_get_state (client) == XFSM_CLIENT_INTERACTING)
+          XfsmClient *cl = lp->data;
+          if (xfsm_client_get_state (cl) == XFSM_CLIENT_INTERACTING)
             {
-              xfsm_client_set_state (client, XFSM_CLIENT_WAITFORINTERACT);
+              xfsm_client_set_state (cl, XFSM_CLIENT_WAITFORINTERACT);
               return;
             }
         }
@@ -1029,13 +1150,13 @@ xfsm_manager_interact_done (XfsmManager *manager,
            lp;
            lp = lp->next)
         {
-          XfsmClient *client = lp->data;
-          if (xfsm_client_get_state (client) != XFSM_CLIENT_WAITFORINTERACT)
+          XfsmClient *cl = lp->data;
+          if (xfsm_client_get_state (cl) != XFSM_CLIENT_WAITFORINTERACT)
             continue;
 
           /* reset all clients that are waiting for interact */
           xfsm_client_set_state (client, XFSM_CLIENT_SAVING);
-          SmsShutdownCancelled (xfsm_client_get_sms_connection (client));
+          SmsShutdownCancelled (xfsm_client_get_sms_connection (cl));
         }
 
         g_signal_emit (manager, signals[SIG_SHUTDOWN_CANCELLED], 0);
@@ -1047,10 +1168,10 @@ xfsm_manager_interact_done (XfsmManager *manager,
            lp;
            lp = lp->next)
         {
-          XfsmClient *client = lp->data;
-          if (xfsm_client_get_state (client) == XFSM_CLIENT_WAITFORINTERACT)
+          XfsmClient *cl = lp->data;
+          if (xfsm_client_get_state (cl) == XFSM_CLIENT_WAITFORINTERACT)
             {
-              xfsm_manager_start_interact (manager, client);
+              xfsm_manager_start_interact (manager, cl);
               break;
             }
         }
@@ -1098,13 +1219,11 @@ xfsm_manager_save_yourself_global (XfsmManager     *manager,
           XfsmShutdownHelper *shutdown_helper;
           GError *error = NULL;
 
-          shutdown_helper = xfsm_shutdown_helper_spawn (&error);
-          if (shutdown_helper == NULL
-              || (!xfsm_shutdown_helper_send_command(shutdown_helper,
-                                                    manager->shutdown_type,
-                                                    &error)
-                  && (error->domain != DBUS_GERROR
-                      || error->code != DBUS_GERROR_NO_REPLY)))
+          shutdown_helper = xfsm_shutdown_helper_new ();
+
+          if (!xfsm_shutdown_helper_send_command (shutdown_helper,
+						  manager->shutdown_type,
+						  &error))
             {
               xfce_message_dialog (NULL, _("Shutdown Failed"),
                                    GTK_STOCK_DIALOG_ERROR,
@@ -1118,8 +1237,8 @@ xfsm_manager_save_yourself_global (XfsmManager     *manager,
             }
 
           /* clean up and return */
-          if (G_LIKELY (shutdown_helper))
-            xfsm_shutdown_helper_destroy (shutdown_helper);
+	  g_object_unref (shutdown_helper);
+
 
           /* at this point, either we failed to suspend/hibernate, or we
            * successfully suspended/hibernated, and we've been woken back
@@ -1164,14 +1283,13 @@ xfsm_manager_save_yourself_global (XfsmManager     *manager,
         {
           XfsmClient *client = lp->data;
           XfsmProperties *properties = xfsm_client_get_properties (client);
+          const gchar *program;
 
           /* xterm's session management is broken, so we won't
            * send a SAVE YOURSELF to xterms */
-          if (properties->program != NULL
-              && strcasecmp (properties->program, "xterm") == 0)
-            {
-              continue;
-            }
+          program = xfsm_properties_get_string (properties, SmProgram);
+          if (program != NULL && strcasecmp (program, "xterm") == 0)
+            continue;
 
           if (xfsm_client_get_state (client) != XFSM_CLIENT_SAVINGLOCAL)
             {
@@ -1317,8 +1435,8 @@ xfsm_manager_close_connection (XfsmManager *manager,
            lp;
            lp = lp->next)
         {
-          XfsmClient *client = lp->data;
-          if (xfsm_client_get_state (client) != XFSM_CLIENT_DISCONNECTED)
+          XfsmClient *cl = lp->data;
+          if (xfsm_client_get_state (cl) != XFSM_CLIENT_DISCONNECTED)
             return;
         }
       
@@ -1432,16 +1550,22 @@ xfsm_manager_perform_shutdown (XfsmManager *manager)
        lp = lp->next)
     {
       XfsmProperties *properties = lp->data;
+      gint            restart_style_hint;
+      gchar         **shutdown_command;
 
-      if (properties->restart_style_hint == SmRestartAnyway
-          && properties->shutdown_command != NULL)
+      restart_style_hint = xfsm_properties_get_uchar (properties,
+                                                      SmRestartStyleHint,
+                                                      SmRestartIfRunning);
+      shutdown_command = xfsm_properties_get_strv (properties, SmShutdownCommand);
+
+      if (restart_style_hint == SmRestartAnyway && shutdown_command != NULL)
         {
           xfsm_verbose ("Client Id = %s, quit already, running shutdown command.\n\n",
                         properties->client_id);
 
-          g_spawn_sync (properties->current_directory,
-                        properties->shutdown_command,
-                        properties->environment,
+          g_spawn_sync (xfsm_properties_get_string (properties, SmCurrentDirectory),
+                        shutdown_command,
+                        xfsm_properties_get_strv (properties, SmEnvironment),
                         G_SPAWN_SEARCH_PATH,
                         NULL, NULL,
                         NULL, NULL,
@@ -1625,7 +1749,8 @@ xfsm_manager_store_session (XfsmManager *manager)
     {
       backup = g_strconcat (manager->session_file, ".bak", NULL);
       unlink (backup);
-      link (manager->session_file, backup);
+      if (link (manager->session_file, backup))
+          g_warning ("Failed to create session file backup");
       g_free (backup);
     }
 
@@ -1653,10 +1778,14 @@ xfsm_manager_store_session (XfsmManager *manager)
     {
       XfsmClient     *client     = lp->data;
       XfsmProperties *properties = xfsm_client_get_properties (client);
+      gint            restart_style_hint;
 
       if (properties == NULL || !xfsm_properties_check (xfsm_client_get_properties (client)))
         continue;
-      if (properties->restart_style_hint == SmRestartNever)
+      restart_style_hint = xfsm_properties_get_uchar (properties,
+                                                      SmRestartStyleHint,
+                                                      SmRestartIfRunning);
+      if (restart_style_hint == SmRestartNever)
         continue;
       
       g_snprintf (prefix, 64, "Client%d_", count);
@@ -1800,9 +1929,9 @@ xfsm_manager_dbus_init (XfsmManager *manager)
 {
   GError *error = NULL;
 
-  manager->dbus_conn = dbus_g_bus_get (DBUS_BUS_SESSION, &error);
+  manager->session_bus = dbus_g_bus_get (DBUS_BUS_SESSION, &error);
 
-  if (G_UNLIKELY (!manager->dbus_conn))
+  if (G_UNLIKELY (!manager->session_bus))
     {
       g_critical ("Unable to contact D-Bus session bus: %s", error ? error->message : "Unknown error");
       if (error)
@@ -1810,11 +1939,11 @@ xfsm_manager_dbus_init (XfsmManager *manager)
       return;
     }
 
-  dbus_g_connection_register_g_object (manager->dbus_conn,
+  dbus_g_connection_register_g_object (manager->session_bus,
                                        "/org/xfce/SessionManager",
                                        G_OBJECT (manager));
 
-  dbus_connection_add_filter (dbus_g_connection_get_connection (manager->dbus_conn),
+  dbus_connection_add_filter (dbus_g_connection_get_connection (manager->session_bus),
                               xfsm_manager_watch_dbus_disconnect,
                               manager, NULL);
 }
@@ -1823,13 +1952,13 @@ xfsm_manager_dbus_init (XfsmManager *manager)
 static void
 xfsm_manager_dbus_cleanup (XfsmManager *manager)
 {
-  if (G_LIKELY (manager->dbus_conn))
+  if (G_LIKELY (manager->session_bus))
     {
-      dbus_connection_remove_filter (dbus_g_connection_get_connection (manager->dbus_conn),
+      dbus_connection_remove_filter (dbus_g_connection_get_connection (manager->session_bus),
                                      xfsm_manager_watch_dbus_disconnect,
                                      manager);
-      dbus_g_connection_unref (manager->dbus_conn);
-      manager->dbus_conn = NULL;
+      dbus_g_connection_unref (manager->session_bus);
+      manager->session_bus = NULL;
     }
 }
 
