@@ -1,4 +1,4 @@
-/* $Id: xfsm-properties.h 4557 2004-05-18 18:07:49Z benny $ */
+/* $Id$ */
 /*-
  * Copyright (c) 2003-2004 Benedikt Meurer <benny@xfce.org>
  * All rights reserved.
@@ -27,7 +27,8 @@
 #include <libxfce4util/libxfce4util.h>
 
 /* GNOME compatibility */
-#define GsmPriority "_GSM_Priority"
+#define GsmPriority     "_GSM_Priority"
+#define GsmDesktopFile  "_GSM_DesktopFile"
 
 #define MAX_RESTART_ATTEMPTS 5
 
@@ -36,19 +37,17 @@ typedef struct _XfsmProperties XfsmProperties;
 struct _XfsmProperties
 {
   guint   restart_attempts;
+  guint   restart_attempts_reset_id;
   
+  guint   startup_timeout_id;
+
+  GPid    pid;
+  guint   child_watch_id;
+
   gchar  *client_id;
   gchar  *hostname;
 
-  gchar **clone_command;
-  gchar  *current_directory;
-  gchar **discard_command;
-  gchar **environment;
-  gint    priority;
-  gchar  *program;
-  gchar **restart_command;
-  gint    restart_style_hint;
-  gchar  *user_id;
+  GTree  *sm_properties;
 };
 
 
@@ -59,24 +58,53 @@ XfsmProperties *xfsm_properties_new (const gchar *client_id,
                                      const gchar *hostname) G_GNUC_PURE;
 void            xfsm_properties_free    (XfsmProperties *properties);
 
-void            xfsm_properties_delete  (XfsmProperties *properties,
-                                         gint            num_props,
-                                         gchar         **prop_names);
 void            xfsm_properties_extract (XfsmProperties *properties,
                                          gint           *num_props,
                                          SmProp       ***props);
-void            xfsm_properties_merge   (XfsmProperties *properties,
-                                         gint            num_props,
-                                         SmProp        **props);
 void            xfsm_properties_store   (XfsmProperties *properties,
-                                      XfceRc         *rc,
-                                      const gchar    *prefix);
+                                         XfceRc         *rc,
+                                         const gchar    *prefix);
 
 XfsmProperties* xfsm_properties_load (XfceRc *rc, const gchar *prefix);
 
 gboolean xfsm_properties_check (const XfsmProperties *properties) G_GNUC_CONST;
 
+G_CONST_RETURN gchar *xfsm_properties_get_string (XfsmProperties *properties,
+                                                  const gchar *property_name);
+gchar **xfsm_properties_get_strv (XfsmProperties *properties,
+                                  const gchar *property_name);
+guchar xfsm_properties_get_uchar (XfsmProperties *properties,
+                                  const gchar *property_name,
+                                  guchar default_value);
+
+const GValue *xfsm_properties_get (XfsmProperties *properties,
+                                   const gchar *property_name);
+
+void xfsm_properties_set_string (XfsmProperties *properties,
+                                 const gchar *property_name,
+                                 const gchar *property_value);
+void xfsm_properties_set_strv (XfsmProperties *properties,
+                               const gchar *property_name,
+                               gchar **property_value);
+void xfsm_properties_set_uchar (XfsmProperties *properties,
+                                const gchar *property_name,
+                                guchar property_value);
+
+gboolean xfsm_properties_set (XfsmProperties *properties,
+                              const gchar *property_name,
+                              const GValue *property_value);
+gboolean xfsm_properties_set_from_smprop (XfsmProperties *properties,
+                                          const SmProp *sm_prop);
+
+gboolean xfsm_properties_remove (XfsmProperties *properties,
+                                 const gchar *property_name);
+
+void xfsm_properties_set_default_child_watch (XfsmProperties *properties);
+
 gint xfsm_properties_compare (const XfsmProperties *a,
                               const XfsmProperties *b) G_GNUC_CONST;
 
+gint xfsm_properties_compare_id (const XfsmProperties *properties,
+                                 const gchar *client_id);
+
 #endif /* !__XFSM_PROPERTIES_H__ */
